@@ -11,7 +11,7 @@
 # This file implements some common TCL routines used for regression
 # testing the SQLite library
 #
-# $Id: tester.tcl,v 1.22 2002/03/06 22:01:37 drh Exp $
+# $Id: tester.tcl,v 1.28 2004/02/14 01:39:50 drh Exp $
 
 # Make sure tclsqlite was compiled correctly.  Abort now with an
 # error message if not.
@@ -40,6 +40,19 @@ if {[sqlite -tcl-uses-utf]} {
   }
 }
 
+# Use the pager codec if it is available
+#
+if {[sqlite -has-codec] && [info command sqlite_orig]==""} {
+  rename sqlite sqlite_orig
+  proc sqlite {args} {
+    if {[llength $args]==2 && [string index [lindex $args 0] 0]!="-"} {
+      lappend args -key {xyzzy}
+    }
+    uplevel 1 sqlite_orig $args
+  }
+}
+
+
 # Create a test database
 #
 catch {db close}
@@ -89,12 +102,12 @@ proc do_test {name cmd expected} {
     puts "\nError: $result"
     incr nErr
     lappend ::failList $name
-    if {$nErr>10} {puts "*** Giving up..."; finalize_testing}
+    if {$nErr>100} {puts "*** Giving up..."; finalize_testing}
   } elseif {[string compare $result $expected]} {
     puts "\nExpected: \[$expected\]\n     Got: \[$result\]"
     incr nErr
     lappend ::failList $name
-    if {$nErr>10} {puts "*** Giving up..."; finalize_testing}
+    if {$nErr>100} {puts "*** Giving up..."; finalize_testing}
   } else {
     puts " Ok"
   }
@@ -158,7 +171,7 @@ proc finish_test {} {
   finalize_testing
 }
 proc finalize_testing {} {
-  global nTest nErr nProb
+  global nTest nErr nProb sqlite_open_file_count
   if {$nErr==0} memleak_check
   catch {db close}
   puts "$nErr errors out of $nTest tests"
@@ -167,6 +180,10 @@ proc finalize_testing {} {
     puts "$nProb probabilistic tests also failed, but this does"
     puts "not necessarily indicate a malfunction."
   }
+  if {$sqlite_open_file_count} {
+    puts "$sqlite_open_file_count files were left open"
+    incr nErr
+  }
   exit [expr {$nErr>0}]
 }
 
@@ -210,6 +227,26 @@ proc execsql2 {sql} {
   return $result
 }
 
+# Use the non-callback API to execute multiple SQL statements
+#
+proc stepsql {dbptr sql} {
+  set sql [string trim $sql]
+  set r 0
+  while {[string length $sql]>0} {
+    if {[catch {sqlite_compile $dbptr $sql sqltail} vm]} {
+      return [list 1 $vm]
+    }
+    set sql [string trim $sqltail]
+    while {[sqlite_step $vm N VAL COL]=="SQLITE_ROW"} {
+      foreach v $VAL {lappend r $v}
+    }
+    if {[catch {sqlite_finalize $vm} errmsg]} {
+      return [list 1 $errmsg]
+    }
+  }
+  return $r
+}
+
 # Delete a file or directory
 #
 proc forcedelete {filename} {
@@ -217,3 +254,11 @@ proc forcedelete {filename} {
     exec rm -rf $filename
   }
 }
+
+# Do an integrity check of the entire database
+#
+proc integrity_check {name} {
+  do_test $name {
+    execsql {PRAGMA integrity_check}
+  } {ok}
+}
